"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LanguageService = void 0;
const graphql_1 = require("graphql");
const picomatch_1 = __importDefault(require("picomatch"));
const graphql_language_service_interface_1 = require("graphql-language-service-interface");
const graphql_language_service_utils_1 = require("graphql-language-service-utils");
const schemaLoader_1 = require("./schemaLoader");
const schemaCache = new Map();
class LanguageService {
    constructor({ parser, schemas, parseOptions, exteralFragmentDefinitions, customValidationRules, }) {
        this._parser = graphql_1.parse;
        this._schemas = [];
        this._schemaCache = schemaCache;
        this._schemaLoader = schemaLoader_1.defaultSchemaLoader;
        this._parseOptions = undefined;
        this._customValidationRules = undefined;
        this._exteralFragmentDefinitionNodes = null;
        this._exteralFragmentDefinitionsString = null;
        this.getCompletion = (uri, documentText, position) => {
            const schema = this.getSchemaForFile(uri);
            if (!documentText || documentText.length < 1 || !(schema === null || schema === void 0 ? void 0 : schema.schema)) {
                return [];
            }
            return graphql_language_service_interface_1.getAutocompleteSuggestions(schema.schema, documentText, position, undefined, this.getExternalFragmentDefinitions());
        };
        this.getDiagnostics = (uri, documentText, customRules) => {
            const schema = this.getSchemaForFile(uri);
            if (!documentText || documentText.length < 1 || !(schema === null || schema === void 0 ? void 0 : schema.schema)) {
                return [];
            }
            return graphql_language_service_interface_1.getDiagnostics(documentText, schema.schema, customRules !== null && customRules !== void 0 ? customRules : this._customValidationRules, false, this.getExternalFragmentDefinitions());
        };
        this.getHover = (uri, documentText, position, options) => {
            const schema = this.getSchemaForFile(uri);
            if (schema && (documentText === null || documentText === void 0 ? void 0 : documentText.length) > 3) {
                return graphql_language_service_interface_1.getHoverInformation(schema.schema, documentText, position, undefined, Object.assign({ useMarkdown: true }, options));
            }
        };
        this.getVariablesJSONSchema = (uri, documentText, options) => {
            const schema = this.getSchemaForFile(uri);
            if (schema && documentText.length > 3) {
                try {
                    const documentAST = this.parse(documentText);
                    const operationFacts = graphql_language_service_utils_1.getOperationASTFacts(documentAST, schema.schema);
                    if (operationFacts && operationFacts.variableToType) {
                        return graphql_language_service_utils_1.getVariablesJSONSchema(operationFacts.variableToType, options);
                    }
                }
                catch (err) { }
            }
            return null;
        };
        this._schemaLoader = schemaLoader_1.defaultSchemaLoader;
        if (schemas) {
            this._schemas = schemas;
        }
        if (parser) {
            this._parser = parser;
            this._cacheSchemas();
        }
        if (parseOptions) {
            this._parseOptions = parseOptions;
        }
        if (customValidationRules) {
            this._customValidationRules = customValidationRules;
        }
        if (exteralFragmentDefinitions) {
            if (Array.isArray(exteralFragmentDefinitions)) {
                this._exteralFragmentDefinitionNodes = exteralFragmentDefinitions;
            }
            else {
                this._exteralFragmentDefinitionsString = exteralFragmentDefinitions;
            }
        }
    }
    _cacheSchemas() {
        this._schemas.forEach(schema => this._cacheSchema(schema));
    }
    _cacheSchema(schemaConfig) {
        const schema = this._schemaLoader(schemaConfig, this.parse);
        return this._schemaCache.set(schemaConfig.uri, Object.assign(Object.assign({}, schemaConfig), { schema }));
    }
    getSchemaForFile(uri) {
        const schema = this._schemas.find(schemaConfig => {
            if (!schemaConfig.fileMatch) {
                return false;
            }
            return schemaConfig.fileMatch.some(glob => {
                const isMatch = picomatch_1.default(glob);
                return isMatch(uri);
            });
        });
        if (schema) {
            const cacheEntry = this._schemaCache.get(schema.uri);
            if (cacheEntry) {
                return cacheEntry;
            }
            const cache = this._cacheSchema(schema);
            return cache.get(schema.uri);
        }
    }
    getExternalFragmentDefinitions() {
        if (!this._exteralFragmentDefinitionNodes &&
            this._exteralFragmentDefinitionsString) {
            const definitionNodes = [];
            try {
                graphql_1.visit(this._parser(this._exteralFragmentDefinitionsString), {
                    FragmentDefinition(node) {
                        definitionNodes.push(node);
                    },
                });
            }
            catch (err) {
                throw Error(`Failed parsing exteralFragmentDefinitions string:\n${this._exteralFragmentDefinitionsString}`);
            }
            this._exteralFragmentDefinitionNodes = definitionNodes;
        }
        return this._exteralFragmentDefinitionNodes;
    }
    updateSchemas(schemas) {
        return __awaiter(this, void 0, void 0, function* () {
            this._schemas = schemas;
            this._cacheSchemas();
        });
    }
    updateSchema(schema) {
        const schemaIndex = this._schemas.findIndex(c => c.uri === schema.uri);
        if (schemaIndex < 0) {
            console.warn('updateSchema could not find a schema in your config by that URI', schema.uri);
            return;
        }
        this._schemas[schemaIndex] = schema;
        this._cacheSchema(schema);
    }
    addSchema(schema) {
        this._schemas.push(schema);
        this._cacheSchema(schema);
    }
    parse(text, options) {
        return this._parser(text, options || this._parseOptions);
    }
}
exports.LanguageService = LanguageService;
//# sourceMappingURL=LanguageService.js.map